<?php

namespace Drupal\cforge\Installer;

use Drupal\mcapi\Entity\Currency;
use Drupal\user\UserInterface;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\node\Entity\NodeType;
use Drupal\node\Entity\Node;
use Drupal\user\Entity\User;
use Drupal\Core\Url;
use Drupal\Core\Render\Element;
use Drupal\Core\Logger\LoggerChannel;
use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\ModuleInstallerInterface;
use Drupal\Core\Session\AccountSwitcherInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Utility\LinkGeneratorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;


ini_set('max_execution_time', 150);

/**
 * Configuration options for each flavour.
 *
 */
class Setup extends FormBase {

  /**
   * @var ModuleInstallerInterface
   */
  private $installer;

  /**
   * @var EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * @var LinkGeneratorInterface
   */
  protected $linkGenerator;

  /**
   * @var LoggerChannel
   */
  protected $logger;

  /**
   * @var AccountSwitcherInterface
   */
  private $accountSwitcher;

  /**
   * Constructor.
   */
  public function __construct(ModuleInstallerInterface $module_installer, LinkGeneratorInterface $link_generator, EntityTypeManagerInterface $entity_type_manager, LoggerChannel $logger_channel, AccountSwitcherInterface $account_switcher) {
    $this->installer = $module_installer;
    $this->linkGenerator = $link_generator;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger_channel;
    $this->accountSwitcher = $account_switcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_installer'),
      $container->get('link_generator'),
      $container->get('entity_type.manager'),
      $container->get('logger.channel.cforge'),
      $container->get('account_switcher')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_setup_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form_state->loadInclude('cforge','install');
    $demo_data_class = cforge_demo_data_class_name();

    $form['data_content'] = [
      '#title' => $this->t('Prepare the site with'),
      '#type' => 'radios',
      '#options' => [
        'none' => t('No content'),
        'import' => $this->t('CSV Import Tools'),
        'upgrade' => $this->t('Upgrade modules'),
        'example' => $this->t('Example data'),
      ],
      '#default_value'  => 'none',
      '#weight' => 3,
      'example' => [
        '#disabled' => !system_rebuild_module_data()['devel_generate'],
      ],
      '#required' => TRUE,
    ];

    $vocab_names = array_keys($demo_data_class::vocabs());
    $form['default_categories'] = array(
      '#title' => $this->t('Preload ad categories'),
      '#description' => $this->t('Can be modified later'),
      '#type' => 'radios',
      '#options' => [0 => $this->t('- None -')] + array_combine($vocab_names, $vocab_names),
      '#default_value' => reset($vocab_names),
      '#weight' => 9,
      '#states' => [
        'visible' => [
          ':input[name=data_content]' => ['value' => 'none'],
        ],
      ],
    );

    $form['currency_type'] = [
      '#title' => $this->t('Main currency type'),
      '#type' => 'radios',
      '#options' => [
        'hours' => $this->t('Hours'),
        'other' => $this->t('Other'),
      ],
      '#default_value' => 'hours',
      '#required' => TRUE,
      '#weight' => 6,
    ];
    $form['currency_name'] = [
      '#title' => $this->t('Currency name'),
      '#description' => $this->t('Use the plural'),
      '#type' => 'textfield',
      '#placeholder' => $this->t('Units'),
      '#weight' => 8,
      '#states' => [
        'visible' => [
          ':input[name="currency_type"]' => ['value' => 'other'],
        ],
      ],
    ];

    $form['direction'] = [
      '#title' => $this->t('Default transaction direction'),
      '#type' => 'radios',
      '#options' => [
        'credit' => $this->t('Credit (push payment)'),
        'bill' => $this->t('Bill (pull payment)'),
        'both' => $this->t('Both (may be confusing for some users)')
      ],
      '#default_value' => 'credit',
      '#weight' => 10,
    ];

    $form['devel'] = [
      '#title' => t('Devel modules'),
      '#type' => 'checkbox',
      '#weight'=> 15
    ];

    $form['actions'] = [
      'submit' => [
        '#type' => 'submit',
        '#value' => 'submit',
      ],
      '#weight' => 20,
    ];
    $form['#submit'][] = '::submitForm';


    foreach (system_rebuild_module_data() as $module_name => $extension) {
      $info = $extension->info;
      if ($info['package'] == 'Cforge features' and empty($info['hidden'])) {
        if (!\Drupal::moduleHandler()->moduleExists($module_name)) {
          $feature_names[$module_name] = new TranslatableMarkup($info['name']);
          $feature_descriptions[$module_name] = new TranslatableMarkup($info['description']);
        }
      }
    }

    $form['features'] = [
      // Hope that yields the current language.
      '#title' => (string) t('Hamlets features'),
      '#type' => 'checkboxes',
      '#options' => $feature_names,
      // Everything enabled by default.
      '#default_value' => array_keys($feature_names),
      '#weight' => 1,
    ];
    foreach ($feature_descriptions as $name => $description) {
      $form['features'][$name]['#description'] = $description;
      $form['features'][$name]['#states']['checked'][':input[name="demo"]']['checked'] = TRUE;
    }

    $alt_login_form = \Drupal::formBuilder()->getForm('Drupal\alt_login\Settings');
    $form['aliases'] = [
      '#title' => $alt_login_form['aliases']['#title'],
      '#type' => 'select',
      '#options' => $alt_login_form['aliases']['#options'],
      '#default_value' => 'address_name',
      '#weight' => 5
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $cforge_settings = $this->configFactory()->getEditable('cforge.settings');
    $cforge_settings->set('member_frontpage', '/news');


    if ($form_state->getValue('direction') == 'credit') {
      if ($display = EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.bill')) {
        $display->setThirdPartySetting('mcapi_forms', 'permission', 'manage mcapi')->save();
      }
    }
    elseif ($form_state->getValue('direction') == 'bill') {
      if ($display = EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.credit')) {
        $display->setThirdPartySetting('mcapi_forms', 'permission', 'manage mcapi')->save();
      }
    }

    $demo_data_class = cforge_demo_data_class_name();
    if ($form_state->getValue('default_categories')) {
      $terms = $this->entityTypeManager
        ->getStorage('taxonomy_term')
        ->loadByProperties(['vid' => SMALLAD_CATEGORIES]);
      // Delete any existing terms in the vocab (one is installed by default)
      foreach ($terms as $term) {
        $term->delete();
      }
      $vocab = Vocabulary::load(SMALLAD_CATEGORIES);
      // Now import the terms into the vocab.
      $this->vocabImportRecursive(
        $vocab,
        $demo_data_class::vocabs()[$form_state->getValue('default_categories')]
      );
      // The hierarchy property.
      $vocab->save();
    }

    // Set up the currency by deleting the others.
    if ($c = Currency::load('veur')) {
      $c->delete();
    }
    if ($form_state->getValue('currency_type') == 'hours') {
      if ($c = Currency::load('cc')) {
        $c->delete();
      }
    }
    if ($form_state->getValue('currency_type') == 'cc') {
      if ($c = Currency::load('hhrs')) {
        $c->delete();
      }
    }
    if ($currs = Currency::loadMultiple()) {
      $curr = reset($currs);
      $curr->name = $form_state->getValue('currency_name') ?: 'Units';
      $symbol = strtoupper(substr($curr->name, 0, 1));
      $curr->format = ["<strong>$symbol</strong>", '9999'];
      $curr->save();
    }
    if ($features = $form_state->getValue('features')) {
      $feature_batch = (new BatchBuilder())
        ->setTitle(t('Installing features'))
        ->setProgressMessage(t('Installing @current out of @total Hamlets features.'))
        ->addOperation([$this->installer, 'install'], [array_filter($features)]);
      batch_set($feature_batch->toArray());
    }

    $modules = [];
    switch ($form_state->getValue('data_content')) {
      case 'import':
        $modules[] = 'cforge_import';
        break;
      case 'upgrade':
        $modules = array_merge($modules, [
          'migrate',
          'migrate_drupal',
          'migrate_drupal_ui',
          'migrate_plus',
          'migrate_tools',
          'migrate_upgrade',
          'options'
        ]);
        break;

      case 'example':
        // Enable example modules and settings
        $this->configFactory()->getEditable('views.settings')
          ->set('ui.show.sql_query.enabled', 1)
          ->set('ui.always_live_preview', 1)
          ->set('ui.show.advanced_column', 1)
          ->set('ui.show.master_display', 1)
          ->save();
        $cforge_settings->set('demo_mode', TRUE);
        $batch_operations[] = ['cf_neighbourhoods_create', [$demo_data_class::$hoods]];

        $this->configFactory()
          ->getEditable('user.settings')
          ->set('register', UserInterface::REGISTER_ADMINISTRATORS_ONLY)
          ->save();
         $this->configFactory()
           ->getEditable('system.logging')
           ->set('error_level', ERROR_REPORTING_DISPLAY_ALL)
           ->save();
        $modules[] = 'help';
        $modules[] = 'devel_generate';
        break;

      default:
        $batch_operations[] = ['cf_neighbourhoods_create', [['Centre']]];
    }
    $cforge_settings->save();

    if ($form_state->getValue('devel')) {
      $modules = array_merge($modules, [
        'devel_php',
        'dblog',
        'field_ui',
        'views_ui',
        'contextual',
        'admin_toolbar_tools',
        'config',
        'config_translation'
      ]);
    }

    if ($modules) {
      $batch_operations[] = [[$this->installer, 'install'], [$modules]];
    }
    if (isset($batch_operations)) {
      $batch = (new BatchBuilder())
        ->setTitle(t('General setup'))
        ->setProgressMessage(t('Installing @current out of @total Hamlets features.'));
      foreach ($batch_operations as $op) {
        $batch->addOperation($op[0], $op[1]);
      }
      batch_set($batch->toArray());
    }

    // Needs to run after the devel_content module is installed
    if ($form_state->getValue('data_content') == 'example') {
      $this->addDemoDataBatch();
    }

    $alt_login_conf = $this->configfactory()->getEditable('alt_login.settings')
      ->set('display_anon', 'Member [user:uid]')
      ->set('display', '[user:address:given_name] [user:address:family_name]')
      ->set('aliases', [$form_state->getvalue('aliases')]);
    if ($form_state->getvalue('aliases') == 'uid') {
      $alt_login_conf
        ->set('display', '[user:address:given_name] [user:address:family_name] ([user:uid])');
    }
    $alt_login_conf->save();


    $form_state->setRedirectUrl(
      Url::fromUserInput($cforge_settings->get('member_frontpage'))
    );
  }

  /**
   * Import the terms in a vocabulary or a level of a vocab.
   */
  public function vocabImportRecursive(Vocabulary $vocab, array $termArray, $parent = 0) {
    $w = $hierarchy = 0;
    foreach (Element::Children($termArray) as $name) {
      $props = [
        'name' => $name,
        'weight' => $w++,
        'vid' => $vocab->id(),
        'parent' => $parent,
        'format' => 'plain_text',
      ];
      $term = Term::Create($props);
      $term->save();
      if ($termArray['#children']) {
        // @todo switch over these lines on 8.2.0
        //$vocab->setHierarchy(VocabularyInterface::HIERARCHY_SINGLE);
        $vocab->setHierarchy(TAXONOMY_HIERARCHY_SINGLE);
        $this->vocabImportRecursive($vocab, $termArray[$name], $term->tid);
      }
    }
  }

  /**
   * utility
   */
  function addDemoDataBatch() {
    $batch_builder = (new BatchBuilder())
      ->setTitle(t('Installing demo data'))
      ->setProgressMessage(t('Processed @current out of @total.'))
      ->setFile('profiles/cforge/src/Setup.php')
      ->setFinishCallback([$this, 'finishDemoImportBatch']);

    // Create 26 users.
    $batch_builder->addOperation(
      [$this, 'generateContent'],
      [
         'hamlets_user', // This is a devel_generate plugin
        [
          'kill' => FALSE,
          'num' => 26,
          'pass' => 'password',
          'roles' => ['trader'],
          'time_range' => strtotime('-2 years'),
        ]
      ]
    );
    $batch_builder->addOperation(
      [$this, 'generateContent'],
      [
        'smallad',
        [
          'kill' => FALSE,
          'num' => 99,
          'type' => 'offer',
          'since' => strtotime('-6 months'),
        ]
      ]
    );
    $batch_builder->addOperation(
      [$this, 'generateContent'],
      [
        'smallad',
        [
          'kill' => FALSE,
          'num' => 50,
          'type' => 'want',
          'since' => strtotime('-6 months'),
        ]
      ]
    );
    $batch_builder->addOperation(
      [$this, 'generateContent'],
      [
        'mcapi_transaction',
        [
          'kill' => FALSE,
          'num' => 75,
          'type' => 'default',
        ]
      ]
    );
    $batch_builder->addOperation(
      [$this, 'generateContent'],
      [
        'mcapi_transaction',
        [
          'kill' => FALSE,
          'num' => 75,
          'type' => 'default',
        ]
      ]
    );
    $batch_builder->addOperation(
      [$this, 'generateContent'],
      [
        'content',
        [
          'kill' => FALSE,
          'num' => 11,
          'node_types' => ['story' => TRUE],
          'max_comments' => 6,
          'title_length' => 8,
        ]
      ]
    );
    if (NodeType::load('event')) {
      $batch_builder->addOperation(
        [$this, 'generateContent'],
        [
          'content',
          [
            'kill' => FALSE,
            'num' => 11,
            'node_types' => ['event' => TRUE],
            'max_comments' => 2,
            'title_length' => 8,
          ]
        ]
      );
    }
    if (NodeType::load('document')) {
      $batch_builder->addOperation(
        [$this, 'generateContent'],
        [
          'content',
          [
            'kill' => FALSE,
            'num' => 11,
            'node_types' => ['document' => TRUE],
            'max_comments' => 0,
            'title_length' => 5,
          ]
        ]
      );
    }
    if (NodeType::load('image')) {
      // Create another 3 galleries.
      $props = ['vid' => 'galleries', 'format' => 'plain_text'];
      Term::create($props + [
        'name' => t('Annnual street party'),
      ])->save();
      Term::create($props + [
        'name' => t('Gurning competition'),
      ])->save();
      Term::create($props + [
        'name' => t('Harvest'),
      ])->save();
      $batch_builder->addOperation(
        [$this, 'generateContent'],
        [
          'content',
          [
            'kill' => FALSE,
            'num' => 50,
            'node_types' => ['image' => TRUE],
            'max_comments' => 4,
            'title_length' => 5,
          ]
        ]
      );
    }
    batch_set($batch_builder->toArray());
  }


  /**
   * Batch callback.
   */
  function generateContent($plugin_id, $args) {
    \Drupal::service('plugin.manager.develgenerate')
      ->createInstance($plugin_id)
      ->generate($args);
  }


  /**
   * Batch callback.
   *
   * Change the dates and owner of event nodes.
   */
  static function finishDemoImportBatch() {
    // Make 2 committee members.
    $uids = \Drupal::entityQuery('user')->condition('uid', 1, '>')->execute();
    shuffle($uids);
    $committee = array_slice($uids, 0, 2);
    foreach ($committee as $uid) {
      $account = User::load($uid);
      $account->addRole('committee'); // equivalent to RID_COMMITTEE
      $account->save();
    }
    $nids = \Drupal::entityQuery('node')
      ->condition('type', ['event', 'image', 'news', 'document'], 'IN')
      ->execute();
    $time = \Drupal::time()->getRequestTime();
    $count = count($nids);
    foreach (Node::loadMultiple($nids) as $nid => $node) {
      $node->setOwnerId($committee[array_rand($committee)]);
      $node->created->value = strtotime('-'.$count-$nid.' days');
      if ($node->hasField('date')) {
        $node->date->value = gmdate(DATETIME_DATETIME_STORAGE_FORMAT, $time + mt_rand(-86400 * 180, 86400 * 180));
      }
      $node->save();
    }
    \Drupal::service('module_installer')->uninstall(['devel_generate']);
  }
}
